unit oppBrowser1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, JvComponentBase, JvDragDrop, Vcl.StdCtrls,
  Vcl.Mask, JvExMask, JvToolEdit, Vcl.ComCtrls, Vcl.ExtCtrls, VirtualTrees, JvExExtCtrls,
  JvSplitter,
  OmniXML;

type
  TfrmOPPBrowser = class(TForm)
    JvDragDrop: TJvDragDrop;
    StatusBar: TStatusBar;
    pnlTop: TPanel;
    inpOppFileName: TJvFilenameEdit;
    lblOppFileName: TLabel;
    vtStructure: TVirtualStringTree;
    splitStructure: TSplitter;
    outSource: TMemo;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure inpOppFileNameChange(Sender: TObject);
    procedure JvDragDropDrop(Sender: TObject; Pos: TPoint; Value: TStrings);
    procedure vtStructureFocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode; Column:
      TColumnIndex);
    procedure vtStructureGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column:
      TColumnIndex; TextType: TVSTTextType; var CellText: string);
  strict private
    FNodeFiles  : PVirtualNode;
    FNodeGlobals: PVirtualNode;
    FXML        : IXMLDocument;
    FXMLFiles   : TStringList;
  private
    procedure LoadFile(const filePath: string);
    procedure LoadFiles(vtNode: PVirtualNode; const filePath: string);
    procedure ReloadFile(const fileName: string);
  public
  end;

var
  frmOPPBrowser: TfrmOPPBrowser;

implementation

uses
  System.RegularExpressions,
  OmniXMLUtils,
  GpVirtualTree;

{$R *.dfm}

const
  CNodeGlobals = -1;
  CNodeFiles = -2;

procedure TfrmOPPBrowser.FormCreate(Sender: TObject);
begin
  FXMLFiles := TStringList.Create;
end;

procedure TfrmOPPBrowser.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FXMLFiles);
end;

procedure TfrmOPPBrowser.inpOppFileNameChange(Sender: TObject);
begin
  if FileExists(inpOppFileName.Text) then
    ReloadFile(inpOppFileName.Text);
end;

procedure TfrmOPPBrowser.JvDragDropDrop(Sender: TObject; Pos: TPoint; Value: TStrings);
begin
  if Value.Count > 0 then
    inpOppFileName.Text := Value[0];
end;

procedure TfrmOPPBrowser.LoadFile(const filePath: string);
var
  source  : string;
  source64: string;
begin
  source64 := GetNodeText(FXML.SelectSingleNode(filePath + '/source'));
  Base64Decode(StringReplace(source64, #$0A, '', [rfReplaceAll]), source);
  outSource.Text := source;
end;

procedure TfrmOPPBrowser.LoadFiles(vtNode: PVirtualNode; const filePath: string);
var
  node: IXMLNode;
begin
  for node in XMLEnumNodes(FXML, filePath+'/name') do begin
    FXMLFiles.Add(filePath + '[name="' + GetNodeText(node) + '"]');
    vtStructure.AddChild(vtNode, pointer(FXMLFiles.Count - 1));
  end;
  vtStructure.Expanded[vtNode] := true;
end;

procedure TfrmOPPBrowser.ReloadFile(const fileName: string);
var
  node: IXMLNode;
begin
  vtStructure.Clear;
  FXMLFiles.Clear;
  FXML := CreateXMLDoc;
  if not XMLLoadFromFile(FXML, fileName) then
    StatusBar.SimpleText := 'Failed to load OPP file'
  else begin
    FNodeGlobals := vtStructure.AddChild(nil, pointer(CNodeGlobals));
    FNodeFiles := vtStructure.AddChild(nil, pointer(CNodeFiles));
    LoadFiles(FNodeGlobals, '//project/globals/file');
    LoadFiles(FNodeFiles, '//project/files/file');
    StatusBar.SimpleText := 'Loaded';
  end;
end;

procedure TfrmOPPBrowser.vtStructureFocusChanged(Sender: TBaseVirtualTree; Node:
  PVirtualNode; Column: TColumnIndex);
var
  fileIndex: integer;
begin
  fileIndex := VTGetNodeDataInt(Sender, Node);
  if fileIndex < 0 then
    Exit;
  LoadFile(FXMLFiles[fileIndex]);
end;

procedure TfrmOPPBrowser.vtStructureGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
var
  nodeData: integer;
begin
  nodeData := VTGetNodeDataInt(Sender, node);
  if nodeData = CNodeGlobals then
    CellText := 'Globals'
  else if nodeData = CNodeFiles then
    CellText := 'Files'
  else
    cellText := TRegEx.Match(FXMLFiles[nodeData], '.*\[name="(.*?)"\]').Groups[1].Value;
end;

end.
